/*
 *	Power Controller v1.00 arm7's source
 */

//----------------------------------------------------------------------------

#include	<nds.h>
#include	<stdio.h>
#include	"powerctl7.h"
#include	"thfifo.h"


static thfifo_t	pow_ch;

//-----------------------------------------------------------------------------

#define	disable_ime(flag)	do{ flag=REG_IME; REG_IME=IME_DISABLE;	}while(0)
#define	restore_ime(flag)	do{ REG_IME=flag;						}while(0)

vu32	power_sleep_request = 0;

//-----------------------------------------------------------------------------
// Power on device
//-----------------------------------------------------------------------------

void
power7on(u32 dev)
{
u16	flag, val;

	SerialWaitBusy();
	disable_ime(flag);

	if (dev & POW_LCD1) {
		val = readPowerManagement(PM_CONTROL_REG);
		val |= PM_BACKLIGHT_TOP;
		writePowerManagement(PM_CONTROL_REG, val);
	}
	if (dev & POW_LCD2) {
		val = readPowerManagement(PM_CONTROL_REG);
		val |= PM_BACKLIGHT_BOTTOM;
		writePowerManagement(PM_CONTROL_REG, val);
	}
	if (dev & POW_SPEAKER) {
		powerON(POWER_SOUND);
	}
	if (dev & POW_SOUND_AMP) {
		val = readPowerManagement(PM_AMPLIFIER_REG);
		val |= PM_AMP_ON;
		writePowerManagement(PM_AMPLIFIER_REG, val);
	}

	restore_ime(flag);
}

//-----------------------------------------------------------------------------
// Power off device
//-----------------------------------------------------------------------------

void
power7off(u32 dev)
{
u16	flag, val;

	SerialWaitBusy();
	disable_ime(flag);

	if (dev & POW_LCD1) {
		val = readPowerManagement(PM_CONTROL_REG);
		val &= ~PM_BACKLIGHT_TOP;
		writePowerManagement(PM_CONTROL_REG, val);
	}
	if (dev & POW_LCD2) {
		val = readPowerManagement(PM_CONTROL_REG);
		val &= ~PM_BACKLIGHT_BOTTOM;
		writePowerManagement(PM_CONTROL_REG, val);
	}
	if (dev & POW_SPEAKER) {
		powerOFF(POWER_SOUND);
	}
	if (dev & POW_SOUND_AMP) {
		val = readPowerManagement(PM_AMPLIFIER_REG);
		val &= ~PM_AMP_ON;
		writePowerManagement(PM_AMPLIFIER_REG, val);
	}
	restore_ime(flag);
}

//-----------------------------------------------------------------------------
// Get status
//-----------------------------------------------------------------------------

int
power7status(void)
{
u16	flag, val;

	SerialWaitBusy();
	disable_ime(flag);
	val = readPowerManagement(PM_BATTERY_REG);
	restore_ime(flag);
	return (int)val;
}

//-----------------------------------------------------------------------------

void
power7ipc(thfifo_t ch, u32 rcv, void *udata)
{
	switch (rcv & POWCMD_MASK) {
		case POWCMD_ON:
			power7on(rcv & POW_MASK);
			break;

		case POWCMD_OFF:
			power7off(rcv & POW_MASK);
			break;

		case POWCMD_SLEEP:
			power_sleep_request = 1;
			break;

		default:
			break;
	}
}

//-----------------------------------------------------------------------------
// Initialize Power Controller.
//-----------------------------------------------------------------------------

void
power7init(void)
{
	// Wait for ARM9's setup

	while ((pow_ch = fifoFindChan(POWIPC_NAME)) < 0);
	while (fifoChanRecvChk(pow_ch) != true);

	fifoChanSetFunc(pow_ch, power7ipc);

	power_sleep_request = 0;
}

//-----------------------------------------------------------------------------
// NDS Sleep Mode
//-----------------------------------------------------------------------------

static void
power7LidUnfoldHandler(void)
{
	//	Do Nothing
}

void
power7sleep(int mode)
{
u16	flag, val;
u32	save_ie;

	switch (mode) {
		case POWSLEEP_FORCE:
				goto sleep;

		case POWSLEEP_CHECK9:
				if (power_sleep_request == 1) {
					goto sleep;
				}
				break;

		default:
				break;
	}
	return;

sleep:

	power_sleep_request = 0;
	irqSet(IRQ_LID, power7LidUnfoldHandler);
	save_ie	= REG_IE;
	REG_IE	= IRQ_LID;								// Enable only IRQ_LID

	{												// LED to SLEEP BLINK
		SerialWaitBusy();
		disable_ime(flag);
		val = readPowerManagement(PM_CONTROL_REG);
		val = (val & ~(0x3 << 4)) | PM_LED_CONTROL(1);	// Sleep Blink
		writePowerManagement(PM_CONTROL_REG, val);
		restore_ime(flag);
	}

	swiSleep();

	{												// LED to NORMAL
		SerialWaitBusy();
		disable_ime(flag);
		val = readPowerManagement(PM_CONTROL_REG);
		val = val & ~(0x3 << 4);
		writePowerManagement(PM_CONTROL_REG, val);
		restore_ime(flag);
	}
	REG_IE	= save_ie;
}


